@@ -12,7 +12,7 @@
 #' @param dir (optional) character vector indicating directory in which files are contained, by default the classic-bugs directory if the installed package is used; to use the current working directory, set this to ""
 #' @param model (optional) one of (1) a character string giving the file name containing the BUGS model code, (2) an R function whose body is the BUGS model code, or (3) the output of \code{nimbleCode}. If a file name, the file can contain a 'var' block and 'data' block in the manner of the JAGS versions of the BUGS examples but should not contain references to other input data files nor a const block. The '.bug' or '.txt' extension can be excluded.
 #' @param data (optional) one of (1) character string giving the file name for an R file providing the input constants and data as R code [assigning individual objects or as a named list] or (2) a named list providing the input constants and data. If neither is provided, the function will look for a file named \code{example}-data including extensions .R, .r, or .txt.
-#' @param inits (optional) (1) character string giving the file name for an R file providing the initial values for parameters as R code [assigning individual objects or as a named list] or (2) a named list providing the values. If neither is provided, the function will look for a file named \code{example}-init or \code{example}-inits including extensions .R, .r, or .txt. 
+#' @param inits (optional) (1) character string giving the file name for an R file providing the initial values for parameters as R code [assigning individual objects or as a named list] or (2) a named list providing the values. If neither is provided, the function will look for a file named \code{example}-init or \code{example}-inits including extensions .R, .r, or .txt.
 #' @param useInits boolean indicating whether to test model with initial values provided via \code{inits}
 #' @param debug logical indicating whether to put the user in a browser for debugging when \code{testBUGSmodel} calls \code{readBUGSmodel}.  Intended for developer use.
 #' @author Christopher Paciorek
@@ -24,17 +24,17 @@
 #' testBUGSmodel('pump')
 #' }
 testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL, inits = NULL, useInits = TRUE, debug = FALSE) {
-  if(requireNamespace('testthat', quietly = TRUE)) { 
+  if(requireNamespace('testthat', quietly = TRUE)) {
     if(!is.null(example) && !is.character(example))
       stop("testBUGSmodel: 'example' argument should be a character vector referring to an existing BUGS example or NULL if provided via the 'model' argument")
     testthat::context(paste0("testing for BUGS example: ", example))
-    
+
     if(is.null(dir)) {
 
       if(is.null(example))
         stop("testBUGSmodel: 'example' is not provided; if not using an example from the BUGS manual examples, set 'dir' to \"\"")
       examplesDir <- system.file("classic-bugs", package = "nimble")
-      
+
       if(file.exists(file.path(examplesDir, 'vol1', example))) {
         vol <- 1
       } else if(file.exists(file.path(examplesDir, 'vol2', example))) {
@@ -42,39 +42,34 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       } else {
         stop(paste0("Example: ", example, " not found in Classic BUGS examples; to use your current working directory or if passing inputs as R objects, set 'dir' to be \"\""))
       }
-      cat("Using example in BUGS example directory of the NIMBLE package.\n")   
+      cat("Using example in BUGS example directory of the NIMBLE package.\n")
       dir <- file.path(examplesDir, paste0('vol', vol), example)
     }
-    
-    if(is.null(model)) model <- example 
-    
+
+    if(is.null(model)) model <- example
+
     if(is.null(model))
       stop("testBUGSmodel: one of 'example' or 'model' must be provided")
-    
-    Rmodel <- readBUGSmodel(model = model, data = data, inits = inits, dir = dir, useInits = useInits, debug = debug, check = FALSE)
-    # setting check to FALSE because check() in some cases (e.g., kidney) causes initialization of values such that inits as passed in do not match values in R or C model and failure of test of whether initial values are maintained 
-    
+
+    Rmodel <- readBUGSmodel(model = model, data = data, inits = inits, dir = dir, useInits = useInits, debug = debug, check = FALSE, calculate = FALSE)
+    # setting check and calculate to FALSE because check() and calculate() in some cases (e.g., ice, kidney) causes initialization of values such that inits as passed in do not match values in R or C model and failure of test of whether initial values are maintained
+
+    skip.file.path <- is.null(dir) || (!is.null(dir) && dir == "") ## previously we could have file.path(NULL, ...) and file.path("",...) cases.  Modifications from here down follow those in readBUGSmodel for Windows compatibility
+
     if(useInits) {
                                         # kludgey as this code is in readBUGSmodel() but no nice way to get it out if I want readBUGSmodel to return the R model; one possibility is to have the inits be embedded in the R model...
       initsFile <- NULL
       if(is.character(inits)) {
-        initsFile <- file.path(dir, inits)
-        if(!file.exists(initsFile)) 
+        initsFile <- if(skip.file.path) inits else file.path(dir, inits)
+        if(!file.exists(initsFile))
           stop("testBUGSmodel: 'inits' input does not reference an existing file.")
       }
       if(is.null(inits)) {
         modelName <- gsub("\\..*", "", model)
-        possibleNames <- c(
-                           file.path(dir, paste0(modelName, "-init.R")),
-                           file.path(dir, paste0(modelName, "-inits.R")),
-                           file.path(dir, paste0(modelName, "-init.txt")),
-                           file.path(dir, paste0(modelName, "-inits.txt")),
-                           file.path(dir, paste0(modelName, "-init")),
-                           file.path(dir, paste0(modelName, "-inits")))
+        possibleNames <- paste0(modelName, c("-init.R", "-inits.R", "-init.txt", "-inits.txt", "-init", "inits"))
         if(!Sys.info()['sysname'] %in% c("Darwin", "Windows")) # UNIX-like is case-sensitive
-          possibleNames <- c(possibleNames,
-                             file.path(dir, paste0(modelName, "-init.r")),
-                             file.path(dir, paste0(modelName, "-inits.r")))
+            possibleNames <- c(possibleNames, paste0(modelName, c('-init.r','-inits.r')))
+        if(!skip.file.path) possibleNames <- file.path(dir, possibleNames)
         fileExistence <- file.exists(possibleNames)
         if(!sum(fileExistence)) {
           stop("testBUGSmodel: 'inits' argument does not reference an existing file.")
@@ -91,7 +86,7 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       }
     }
     if(useInits && is.null(inits))
-      warning("testBUGSmodel: 'useInits' is TRUE but 'inits' is NULL and could not find file of initial values in directory provided; proceeding without initial values.")  
+      warning("testBUGSmodel: 'useInits' is TRUE but 'inits' is NULL and could not find file of initial values in directory provided; proceeding without initial values.")
                                         # for C model
 ##    Cmodel <- compileBUGSmodel(Rmodel)
     project <- nimbleProjectClass(NULL, name = 'foo')
@@ -100,7 +95,7 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
     nodeNames <- Rmodel$getNodeNames()
     detNodeNames <- Rmodel$getNodeNames(determOnly = TRUE)
                                         #varNames <- Rmodel$getVarNames()
-    
+
     set.seed(0)
                                         # simulate/calculate in topological order
     for(nodeName in nodeNames) {
@@ -110,7 +105,7 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       if(!(nodeName %in% detNodeNames && varName %in% names(inits))) # don't overwrite det nodes that have init values
         calculate(Rmodel, nodeName)
     }
-    
+
     set.seed(0)
     for(nodeName in nodeNames) {
       varName <- gsub("\\[.*\\]", "", nodeName)
@@ -138,7 +133,7 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       for(nodeName in nodeNames) {
           Rvals <- Rmodel[[nodeName]]
           Cvals <- Cmodel[[nodeName]]
-          attributes(Rvals) <- attributes(Cvals) <- NULL        
+          attributes(Rvals) <- attributes(Cvals) <- NULL
         testthat::expect_that(Rvals, testthat::equals(Cvals), info = paste0('Unexpected result for variable ', nodeName))
       }
     })
@@ -146,12 +141,12 @@ testBUGSmodel <- function(example = NULL, dir = NULL, model = NULL, data = NULL,
       for(nodeName in nodeNames)  {
         Rvals <- getLogProb(Rmodel, nodeName)
         Cvals <- getLogProb(Cmodel, nodeName)
-        attributes(Rvals) <- attributes(Cvals) <- NULL        
+        attributes(Rvals) <- attributes(Cvals) <- NULL
         testthat::expect_that(Rvals, testthat::equals(Cvals), info = paste0('Unexpected result for variable ', nodeName))
       }
     })
 
-    dyn.unload(project$cppProjects[[1]]$getSOName())
+    if(.Platform$OS.type != 'windows') dyn.unload(project$cppProjects[[1]]$getSOName())
     # this works to avoid having too many DLLs, but gives segfault when one quits R afterwards
     if(debug) browser()
   } else warning("testBUGSmodel: testthat package is required.")
