@@ -6,16 +6,20 @@
 
 nodeFunctionVector <- setRefClass(
     Class = 'nodeFunctionVector',
-    fields = list(model = 'ANY',
-               	   gids = 'ANY'),
+    fields = list(
+        model = 'ANY',
+        gids = 'ANY',
+        indexingInfo = 'ANY'
+                  ),
                #   nodes = 'ANY'),
                #   nodeFunctionRefClassObjects = 'ANY'),
     methods = list(
         initialize = function(model, nodeNames, excludeData = FALSE, env = parent.frame()) {
             model <<- model
-            if(length(nodeNames) == 0)
+            if(length(nodeNames) == 0) {
             	gids <<- numeric(0)
-            else{
+                indexingInfo <<- list(declIDs = integer(), rowIndices = integer())
+            } else {
                 if(is.numeric(nodeNames)) 		#In case we start with graph ids instead of names
                     temp_gids <- unique(sort(nodeNames, FALSE),
                                                   FALSE,
@@ -30,16 +34,17 @@ nodeFunctionVector <- setRefClass(
             # temp_gids <- .Internal(unique(.Internal(sort(model$modelDef$nodeName2GraphIDs(nodeNames), FALSE)), F,F,NA))))
     	        if(excludeData == TRUE)
                     temp_gids <- temp_gids[!model$isDataFromGraphID(temp_gids)]
-    	        gids <<- temp_gids
+                gids <<- temp_gids ## old nodeFun system
+                indexingInfo <<- model$modelDef$graphIDs2indexedNodeInfo(temp_gids) ## new nodeFun system
             }
         },
         getNodeNames = function(){ ## not used anywhere. provided only for debugging/inspection
         	model$expandNodeNames(gids)	
         },
-        show = function() cat(paste0('nodeFunctionVector: ', paste(nodes, collapse=', '), '\n'))
+        show = function() {
+            cat('nodeFunctionVector: \n')
+            print(cbind(indexingInfo$declIDs, indexingInfo$unrolledIndicesMatrixRows))
+        }
     )
 )
 
-getNodeFxnPtrs <- function(cModel){	
-    lapply( cModel$nodes, `[[`, '.basePtr' )
-}
