@@ -32,15 +32,87 @@ class NodeVectorClass {
   virtual ~NodeVectorClass() {};
 };
 
+class oneNodeUseInfo {
+ public:
+  nodeFun *nodeFunPtr;
+  useInfoForIndexedNodeInfo useInfo;
+  oneNodeUseInfo(nodeFun *nFP, int firstIndex) {
+    nodeFunPtr = nFP;
+    useInfo.indicesForIndexedNodeInfo.push_back(firstIndex);
+  }
+};
+
+class NodeVectorClassNew {
+ public:
+  vector<oneNodeUseInfo> useInfoVec;
+  vector<oneNodeUseInfo> &getUseInfoVec() {return(useInfoVec);}
+};
+
 ///// Using NodeVectors:
 // utilities for calling node functions from a vector of node pointers
 // see .cpp file for definitions
-double calculate(NodeVectorClass &nodes);
-double calculateDiff(NodeVectorClass &nodes);
-double calculateFaster(NodeVectorClass &nodes);
-double getLogProb(NodeVectorClass &nodes);
-void simulate(NodeVectorClass &nodes);
+double calculate(NodeVectorClassNew &nodes);
+double calculate(NodeVectorClassNew &nodes, int iNodeFunction);
+double calculateDiff(NodeVectorClassNew &nodes);
+double calculateDiff(NodeVectorClassNew &nodes, int iNodeFunction);
+double getLogProb(NodeVectorClassNew &nodes);
+double getLogProb(NodeVectorClassNew &nodes, int iNodeFunction);
+void simulate(NodeVectorClassNew &nodes);
+void simulate(NodeVectorClassNew &nodes, int iNodeFunction);
 
+// ideas on efficiency
+/* ## could propagate const-ness through getParam_0D_double_block etc. - that helped */
+/* ## could pull getParam implementations back to .h for inlining. */
+/*  ## could make a const version of operator[] and operator() for NimArray's */
+/*  ## the getParam_..._block is not needed. -skipping this didn't help   */
+/*  ## could do this all on a new branch of newNodeFxn to be able to compare */
+/*  ## instead of the extra argument with default 0, use overloaded versions - might have helped - didn't compare carefully */
+/*  ## try making all calculate, simulate etc. const */
+ 
+
+// for these, if there is use of iNodeFunction, it is generated directly from cppOutputGetParam
+//getParam_0D
+inline double getParam_0D_double(int paramID, const oneNodeUseInfo &useInfo) {
+  return(useInfo.nodeFunPtr->getParam_0D_double_block(paramID, useInfo.useInfo));
+  //return(useInfo.nodeFunPtr->getParam_0D_double(paramID, (*(useInfo.nodeFunPtr->getIndexedNodeInfoTablePtr()))[ useInfo.useInfo.indicesForIndexedNodeInfo[0] ] ) );
+}
+inline double getParam_0D_double(int paramID, const oneNodeUseInfo &useInfo, int iNodeFunction) { 
+  /* iNodeFunction sometimes needs to be generated in a call even if not needed */
+  /* but we want to avoid compiled warnings about an unused argument */
+  /* the following line of code tries to make the compiler think iNodeFunction will be used */
+  if(iNodeFunction) paramID += 0;
+  return(useInfo.nodeFunPtr->getParam_0D_double_block(paramID, useInfo.useInfo));
+}
+template<typename paramIDtype>
+inline double getParam_0D_double(const paramIDtype &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction) {
+return(useInfo.nodeFunPtr->getParam_0D_double_block(paramID[iNodeFunction], useInfo.useInfo));
+}
+
+//getParam_1D
+NimArr<1, double> getParam_1D_double(int paramID, const oneNodeUseInfo &useInfo, int iNodeFunction = 0);
+
+/* template<typename paramIDtype> */
+/* NimArr<1, double> getParam_1D_double(const paramIDtype &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction); */
+
+template<class paramIDtype>
+NimArr<1, double> getParam_1D_double(const paramIDtype &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction) {
+  return(useInfo.nodeFunPtr->getParam_1D_double_block(paramID[iNodeFunction], useInfo.useInfo));
+}
+
+//extern template NimArr<1, double> getParam_1D_double<NimArr<1, int> >(const NimArr<1, int> &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction);
+//extern template NimArr<1, double> getParam_1D_double<NimArr<1, double> >(const NimArr<1, double> &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction);
+
+//getParam_2D
+NimArr<2, double> getParam_2D_double(int paramID, const oneNodeUseInfo &useInfo, int iNodeFunction = 0);
+/* template<typename paramIDtype> */
+/* NimArr<2, double> getParam_2D_double(const paramIDtype &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction); */
+template<class paramIDtype>
+NimArr<2, double> getParam_2D_double(const paramIDtype &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction) {
+  return(useInfo.nodeFunPtr->getParam_2D_double_block(paramID[iNodeFunction], useInfo.useInfo));
+}
+
+//extern template NimArr<2, double> getParam_2D_double<NimArr<1, int> >(const NimArr<1, int> &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction);
+//extern template NimArr<2, double> getParam_2D_double<NimArr<1, double> >(const NimArr<1, double> &paramID, const oneNodeUseInfo &useInfo, int iNodeFunction);
 
 /////////////////////
 // new version of variable accessors using maps (offset and strided windows into multivariate objects (NimArr<>s) )
@@ -531,11 +603,11 @@ extern "C" {
   SEXP getModelAccessorValues(SEXP accessor);
   SEXP getMVAccessorValues(SEXP accessor);
 
-  SEXP newNodeFxnVector(SEXP size);
+  //SEXP newNodeFxnVector(SEXP size);
   SEXP setNodeModelPtr(SEXP nodeFxnPtr, SEXP modelElementPtr, SEXP nodeElementName);
-  SEXP resizeNodeFxnVector(SEXP nodeFxnVecPtr, SEXP size);
-  SEXP addNodeFun(SEXP nVPtr, SEXP nFPtr, SEXP addAtEnd, SEXP index);
-  SEXP removeNodeFun(SEXP rPtr, SEXP index, SEXP removeAll);
+  //SEXP resizeNodeFxnVector(SEXP nodeFxnVecPtr, SEXP size);
+  //  SEXP addNodeFun(SEXP nVPtr, SEXP nFPtr, SEXP addAtEnd, SEXP index);
+  //SEXP removeNodeFun(SEXP rPtr, SEXP index, SEXP removeAll);
 	
   SEXP newManyVariableAccessor(SEXP size);
   SEXP addSingleVariableAccessor(SEXP MVAPtr, SEXP SVAPtr, SEXP addAtEnd, SEXP index);
@@ -555,13 +627,15 @@ extern "C" {
   SEXP varAndIndices2mapParts(SEXP SvarAndIndicesExtPtr, SEXP Ssizes, SEXP SnDim);
   SEXP var2mapParts(SEXP Sinput, SEXP Ssizes, SEXP SnDim);
   
-  SEXP populateNodeFxnVector(SEXP nodeFxnVec, SEXP nodeNames, SEXP );
+  //  SEXP populateNodeFxnVector(SEXP nodeFxnVec, SEXP nodeNames, SEXP );
   SEXP populateNodeFxnVector_byGID(SEXP SnodeFxnVec, SEXP S_GIDs, SEXP SnumberedObj);
+  SEXP populateNodeFxnVectorNew_byDeclID(SEXP SnodeFxnVec, SEXP S_GIDs, SEXP SnumberedObj, SEXP S_ROWINDS);
+  SEXP populateIndexedNodeInfoTable(SEXP StablePtr, SEXP StableContents);
   SEXP populateValueMapAccessorsFromNodeNames(SEXP StargetPtr, SEXP SnodeNames, SEXP SsizesAndNdims, SEXP SModelOrModelValuesPtr );
   SEXP populateValueMapAccessors(SEXP StargetPtr, SEXP SsourceList, SEXP SModelOrModelValuesPtr );
   //	SEXP populateNumberedObject_withSingleModelValuesAccessors(SEXP mvPtr, SEXP varName, SEXP beginIndex, SEXP varLength, SEXP curRow, SEXP SnumbObj);
   SEXP populateNumberedObject_withSingleModelValuesAccessors(SEXP mvPtr, SEXP varName, SEXP GIDs, SEXP curRow, SEXP SnumbObj);
-  SEXP populateModelValuesAccessors_byGID(SEXP SmodelValuesAccessorVector, SEXP S_GIDs, SEXP SnumberedObj);
+  //    SEXP populateModelValuesAccessors_byGID(SEXP SmodelValuesAccessorVector, SEXP S_GIDs, SEXP SnumberedObj);
 
   SEXP populateCopierVector(SEXP ScopierVector, SEXP SfromPtr, SEXP StoPtr, SEXP SintIsFromMV, SEXP SintIsToMV);
   
